/*
 * This file is part of the OWL API.
 *
 * The contents of this file are subject to the LGPL License, Version 3.0.
 *
 * Copyright (C) 2011, The University of Manchester
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *
 * Alternatively, the contents of this file may be used under the terms of the Apache License, Version 2.0
 * in which case, the provisions of the Apache License Version 2.0 are applicable instead of those above.
 *
 * Copyright 2011, University of Manchester
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.semanticweb.owlapi.io;

import java.util.ArrayList;
import java.util.List;

import org.openrdf.rio.RDFFormat;
import org.semanticweb.owlapi.formats.OWLOntologyFormatFactory;

import com.github.ansell.abstractserviceloader.AbstractServiceLoader;


/**
 * Author: Matthew Horridge<br>
 * The University Of Manchester<br>
 * Bio-Health Informatics Group<br>
 * Date: 15-Nov-2006<br><br>
 * <p/>
 * The <code>OWLParserFactoryRegistry</code> provides a central point for
 * the registration of parser factories that create parsers to parse OWL
 * ontologies.  The registry is typically used by at least one type of ontology
 * factory for loading ontologies whose concrete representations are contained
 * in some kind of document.
 */
public class OWLParserFactoryRegistry extends AbstractServiceLoader<OWLOntologyFormatFactory, OWLParserFactory> {

    private static final OWLParserFactoryRegistry instance = new OWLParserFactoryRegistry();

    public OWLParserFactoryRegistry() {
        super(OWLParserFactory.class);
    }


    /**@return the parser factory registry*/
    public static OWLParserFactoryRegistry getInstance() {
        return instance;
    }


    /**
     * clear all registered parser factories
     */
    public void clearParserFactories() {
        this.clear();
    }


    /**
     * @return the list of parsers - changes will not be backed by the factory
     */
    public List<OWLParserFactory> getParserFactories() {
        ArrayList<OWLParserFactory> all = new ArrayList<OWLParserFactory>(this.getAll());
        
        ArrayList<OWLParserFactory> list = new ArrayList<OWLParserFactory>(all.size());
        
        // HACK: The execution strategy used by OWLAPI, where all parsers are tried 
        // and the first to succeed wins, is broken by the SPI random access, so need 
        // to at least make sure that the RDF/XML parser comes before the OWLXML parser, 
        // as they are both the same format and some RDF/XML documents seem to be detected 
        // as syntactically valid OWLXML
        for(OWLParserFactory nextFactory : all)
        {
            if(nextFactory.getFormatFactory().handlesMimeType(RDFFormat.RDFXML.getDefaultMIMEType()))
            {
                list.add(nextFactory);
            }
        }
        
        // Then add all of the parsers that did not match above
        for(OWLParserFactory nextFactory : all)
        {
            if(!nextFactory.getFormatFactory().handlesMimeType(RDFFormat.RDFXML.getDefaultMIMEType()))
            {
                list.add(nextFactory);
            }
        }
        
        return list;
    }

    public OWLParserFactory getParserFactory(String mimeType)
    {
        ArrayList<OWLParserFactory> all = new ArrayList<OWLParserFactory>(this.getAll());
        
        for(OWLParserFactory nextFactory : all)
        {
            if(nextFactory.getFormatFactory().handlesMimeType(mimeType))
            {
                return nextFactory;
            }
        }
        
        return null;
    }
    
    public OWLParserFactory getParserFactory(OWLOntologyFormatFactory referenceFactory)
    {
        ArrayList<OWLParserFactory> all = new ArrayList<OWLParserFactory>(this.getAll());
        
        for(OWLParserFactory nextFactory : all)
        {
            if(nextFactory.getFormatFactory().equals(referenceFactory))
            {
                return nextFactory;
            }
        }
        
        return null;
    }
    
    /**
     * @param parserFactory the parser factory to register
     */
    public void registerParserFactory(OWLParserFactory parserFactory) {
        this.add(parserFactory);
    }


    /**
     * @param parserFactory the parser factory to remove
     */
    public void unregisterParserFactory(OWLParserFactory parserFactory) {
        this.remove(parserFactory);
    }


    @Override
    protected OWLOntologyFormatFactory getKey(OWLParserFactory service) {
        return service.getFormatFactory();
    }
}
